@@ -27,7 +27,7 @@
 #include <mutex>
 #include <condition_variable>
 
-#include "apputil.hpp"  // CreateAddrInet
+#include "apputil.hpp"  // CreateAddr
 #include "uriparser.hpp"  // UriParser
 #include "socketoptions.hpp"
 #include "logsupport.hpp"
@@ -60,6 +60,14 @@ testmedia.cpp
 
 using namespace std;
 
+const srt_logging::LogFA SRT_LOGFA_APP = 10;
+namespace srt_logging
+{
+Logger applog(SRT_LOGFA_APP, srt_logger_config, "TUNNELAPP");
+}
+
+using srt_logging::applog;
+
 class Medium
 {
     static int s_counter;
@@ -89,9 +97,10 @@ protected:
     mutex access; // For closing
 
     template <class DerivedMedium, class SocketType>
-    static Medium* CreateAcceptor(DerivedMedium* self, const sockaddr_in& sa, SocketType sock, size_t chunk)
+    static Medium* CreateAcceptor(DerivedMedium* self, const sockaddr_any& sa, SocketType sock, size_t chunk)
     {
-        DerivedMedium* m = new DerivedMedium(UriParser(self->type() + string("://") + SockaddrToString((sockaddr*)&sa)), chunk);
+        string addr = sockaddr_any(sa.get(), sizeof sa).str();
+        DerivedMedium* m = new DerivedMedium(UriParser(self->type() + string("://") + addr), chunk);
         m->m_socket = sock;
         return m;
     }
@@ -111,14 +120,21 @@ public:
 
     virtual const char* type() = 0;
     virtual bool IsOpen() = 0;
-    virtual void Close() = 0;
+    virtual void CloseInternal() = 0;
+
+    void Close()
+    {
+        m_open = false;
+        m_broken = true;
+        CloseInternal();
+    }
     virtual bool End() = 0;
 
     virtual int ReadInternal(char* output, int size) = 0;
     virtual bool IsErrorAgain() = 0;
 
-    ReadStatus Read(ref_t<bytevector> output);
-    virtual void Write(ref_t<bytevector> portion) = 0;
+    ReadStatus Read(bytevector& output);
+    virtual void Write(bytevector& portion) = 0;
 
     virtual void CreateListener() = 0;
     virtual void CreateCaller() = 0;
@@ -225,6 +241,13 @@ public:
         // If this thread is already stopped, don't stop.
         if (thr.joinable())
         {
+            LOGP(applog.Debug, "Engine::Stop: Closing media:");
+            // Close both media as a hanged up reading thread
+            // will block joining.
+            media[0]->Close();
+            media[1]->Close();
+
+            LOGP(applog.Debug, "Engine::Stop: media closed, joining engine thread:");
             if (thr.get_id() == std::this_thread::get_id())
             {
                 // If this is this thread which called this, no need
@@ -232,10 +255,12 @@ public:
                 // You must, however, detach yourself, or otherwise the thr's
                 // destructor would kill the program.
                 thr.detach();
+                LOGP(applog.Debug, "DETACHED.");
             }
             else
             {
                 thr.join();
+                LOGP(applog.Debug, "Joined.");
             }
         }
     }
@@ -334,14 +359,14 @@ void Engine::Worker()
         try
         {
             which_medium = media[DIR_IN];
-            rdst = media[DIR_IN]->Read(Ref(outbuf));
+            rdst = media[DIR_IN]->Read((outbuf));
             switch (rdst)
             {
             case Medium::RD_DATA:
                 {
                     which_medium = media[DIR_OUT];
                     // We get the data, write them to the output
-                    media[DIR_OUT]->Write(Ref(outbuf));
+                    media[DIR_OUT]->Write((outbuf));
                 }
                 break;
 
@@ -350,6 +375,10 @@ void Engine::Worker()
                 throw Medium::ReadEOF("");
 
             case Medium::RD_AGAIN:
+                // Theoreticall RD_AGAIN should not be reported
+                // because it should be taken care of internally by
+                // repeated sending - unless we get m_broken set.
+                // If it is, however, it should be handled just like error.
             case Medium::RD_ERROR:
                 status = -1;
                 Medium::Error("Error while reading");
@@ -357,7 +386,7 @@ void Engine::Worker()
         }
         catch (Medium::ReadEOF&)
         {
-            Verb() << "EOF. Exitting engine.";
+            Verb() << "EOF. Exiting engine.";
             break;
         }
         catch (Medium::TransmissionError& er)
@@ -396,7 +425,7 @@ public:
     bool End() override { return m_eof; }
     bool Broken() override { return m_broken; }
 
-    void Close() override
+    void CloseInternal() override
     {
         Verb() << "Closing SRT socket for " << uri();
         lock_guard<mutex> lk(access);
@@ -410,7 +439,7 @@ public:
     virtual int ReadInternal(char* output, int size) override;
     virtual bool IsErrorAgain() override;
 
-    virtual void Write(ref_t<bytevector> portion) override;
+    virtual void Write(bytevector& portion) override;
     virtual void CreateListener() override;
     virtual void CreateCaller() override;
     virtual unique_ptr<Medium> Accept() override;
@@ -458,7 +487,7 @@ public:
 
     enum { DEF_SEND_FLAG = 0 };
 
-#elif defined(LINUX) || defined(GNU)
+#elif defined(LINUX) || defined(GNU) || defined(CYGWIN)
     static int tcp_close(int socket)
     {
         return ::close(socket);
@@ -466,7 +495,7 @@ public:
 
     enum { DEF_SEND_FLAG = MSG_NOSIGNAL };
 
-#elif defined(BSD) || defined(OSX) || (TARGET_OS_IOS == 1) || (TARGET_OS_TV == 1)
+#else
     static int tcp_close(int socket)
     {
         return ::close(socket);
@@ -480,7 +509,7 @@ public:
     bool End() override { return m_eof; }
     bool Broken() override { return m_broken; }
 
-    void Close() override
+    void CloseInternal() override
     {
         Verb() << "Closing TCP socket for " << uri();
         lock_guard<mutex> lk(access);
@@ -493,7 +522,7 @@ public:
     virtual const char* type() override { return "tcp"; }
     virtual int ReadInternal(char* output, int size) override;
     virtual bool IsErrorAgain() override;
-    virtual void Write(ref_t<bytevector> portion) override;
+    virtual void Write(bytevector& portion) override;
     virtual void CreateListener() override;
     virtual void CreateCaller() override;
     virtual unique_ptr<Medium> Accept() override;
@@ -574,9 +603,9 @@ void SrtMedium::CreateListener()
 
     ConfigurePre();
 
-    sockaddr_in sa = CreateAddrInet(m_uri.host(), m_uri.portno());
+    sockaddr_any sa = CreateAddr(m_uri.host(), m_uri.portno());
 
-    int stat = srt_bind(m_socket, (sockaddr*)&sa, sizeof sa);
+    int stat = srt_bind(m_socket, sa.get(), sizeof sa);
 
     if ( stat == SRT_ERROR )
     {
@@ -598,12 +627,13 @@ void TcpMedium::CreateListener()
 {
     int backlog = 5; // hardcoded!
 
-    m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+
+    sockaddr_any sa = CreateAddr(m_uri.host(), m_uri.portno());
+
+    m_socket = socket(sa.get()->sa_family, SOCK_STREAM, IPPROTO_TCP);
     ConfigurePre();
 
-    sockaddr_in sa = CreateAddrInet(m_uri.host(), m_uri.portno());
-
-    int stat = ::bind(m_socket, (sockaddr*)&sa, sizeof sa);
+    int stat = ::bind(m_socket, sa.get(), sa.size());
 
     if (stat == -1)
     {
@@ -623,15 +653,19 @@ void TcpMedium::CreateListener()
 
 unique_ptr<Medium> SrtMedium::Accept()
 {
-    sockaddr_in sa;
-    int salen = sizeof sa;
-    SRTSOCKET s = srt_accept(m_socket, (sockaddr*)&sa, &salen);
+    sockaddr_any sa;
+    SRTSOCKET s = srt_accept(m_socket, (sa.get()), (&sa.len));
     if (s == SRT_ERROR)
     {
         Error(UDT::getlasterror(), "srt_accept");
     }
 
     ConfigurePost(s);
+
+    // Configure 1s timeout
+    int timeout_1s = 1000;
+    srt_setsockflag(m_socket, SRTO_RCVTIMEO, &timeout_1s, sizeof timeout_1s);
+
     unique_ptr<Medium> med(CreateAcceptor(this, sa, s, m_chunk));
     Verb() << "accepted a connection from " << med->uri();
 
@@ -640,14 +674,23 @@ unique_ptr<Medium> SrtMedium::Accept()
 
 unique_ptr<Medium> TcpMedium::Accept()
 {
-    sockaddr_in sa;
-    socklen_t salen = sizeof sa;
-    int s = ::accept(m_socket, (sockaddr*)&sa, &salen);
+    sockaddr_any sa;
+    int s = ::accept(m_socket, (sa.get()), (&sa.syslen()));
     if (s == -1)
     {
         Error(errno, "accept");
     }
 
+    // Configure 1s timeout
+    timeval timeout_1s { 1, 0 };
+    int st = setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout_1s, sizeof timeout_1s);
+    timeval re;
+    socklen_t size = sizeof re;
+    int st2 = getsockopt(s, SOL_SOCKET, SO_RCVTIMEO, (char*)&re, &size);
+
+    LOGP(applog.Debug, "Setting SO_RCVTIMEO to @", m_socket, ": ", st == -1 ? "FAILED" : "SUCCEEDED",
+            ", read-back value: ", st2 == -1 ? int64_t(-1) : (int64_t(re.tv_sec)*1000000 + re.tv_usec)/1000, "ms");
+
     unique_ptr<Medium> med(CreateAcceptor(this, sa, s, m_chunk));
     Verb() << "accepted a connection from " << med->uri();
 
@@ -670,37 +713,80 @@ void TcpMedium::CreateCaller()
 
 void SrtMedium::Connect()
 {
-    sockaddr_in sa = CreateAddrInet(m_uri.host(), m_uri.portno());
+    sockaddr_any sa = CreateAddr(m_uri.host(), m_uri.portno());
 
-    int st = srt_connect(m_socket, (sockaddr*)&sa, sizeof sa);
+    int st = srt_connect(m_socket, sa.get(), sizeof sa);
     if (st == SRT_ERROR)
         Error(UDT::getlasterror(), "srt_connect");
 
     ConfigurePost(m_socket);
+
+    // Configure 1s timeout
+    int timeout_1s = 1000;
+    srt_setsockflag(m_socket, SRTO_RCVTIMEO, &timeout_1s, sizeof timeout_1s);
 }
 
 void TcpMedium::Connect()
 {
-    sockaddr_in sa = CreateAddrInet(m_uri.host(), m_uri.portno());
+    sockaddr_any sa = CreateAddr(m_uri.host(), m_uri.portno());
 
-    int st = ::connect(m_socket, (sockaddr*)&sa, sizeof sa);
+    int st = ::connect(m_socket, sa.get(), sa.size());
     if (st == -1)
         Error(errno, "connect");
 
     ConfigurePost(m_socket);
+
+    // Configure 1s timeout
+    timeval timeout_1s { 1, 0 };
+    setsockopt(m_socket, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout_1s, sizeof timeout_1s);
 }
 
-int SrtMedium::ReadInternal(char* buffer, int size)
+int SrtMedium::ReadInternal(char* w_buffer, int size)
 {
-    int st = srt_recv(m_socket, buffer, size);
-    if (st == SRT_ERROR)
-        return -1;
+    int st = -1;
+    do
+    {
+        st = srt_recv(m_socket, (w_buffer), size);
+        if (st == SRT_ERROR)
+        {
+            int syserr;
+            if (srt_getlasterror(&syserr) == SRT_EASYNCRCV && !m_broken)
+                continue;
+        }
+        break;
+
+    } while (true);
+
     return st;
 }
 
-int TcpMedium::ReadInternal(char* buffer, int size)
+int TcpMedium::ReadInternal(char* w_buffer, int size)
 {
-    return ::recv(m_socket, buffer, size, 0);
+    int st = -1;
+    LOGP(applog.Debug, "TcpMedium:recv @", m_socket, " - begin");
+    do
+    {
+        st = ::recv(m_socket, (w_buffer), size, 0);
+        if (st == -1)
+        {
+            if ((errno == EAGAIN || errno == EWOULDBLOCK))
+            {
+                if (!m_broken)
+                {
+                    LOGP(applog.Debug, "TcpMedium: read:AGAIN, repeating");
+                    continue;
+                }
+                LOGP(applog.Debug, "TcpMedium: read:AGAIN, not repeating - already broken");
+            }
+            else
+            {
+                LOGP(applog.Debug, "TcpMedium: read:ERROR: ", errno);
+            }
+        }
+        break;
+    } while (true);
+    LOGP(applog.Debug, "TcpMedium:recv @", m_socket, " - result: ", st);
+    return st;
 }
 
 bool SrtMedium::IsErrorAgain()
@@ -721,19 +807,17 @@ bool TcpMedium::IsErrorAgain()
 // This will cause the worker loop to redirect to Write immediately
 // thereafter and possibly will flush out the remains of the buffer.
 // It's still possible that the buffer won't be completely purged
-Medium::ReadStatus Medium::Read(ref_t<bytevector> r_output)
+Medium::ReadStatus Medium::Read(bytevector& w_output)
 {
-    bytevector& output = *r_output;
-
     // Don't read, but fake that you read
-    if (output.size() > m_chunk)
+    if (w_output.size() > m_chunk)
     {
         Verb() << "BUFFER EXCEEDED";
         return RD_DATA;
     }
 
     // Resize to maximum first
-    size_t shift = output.size();
+    size_t shift = w_output.size();
     if (shift && m_eof)
     {
         // You have nonempty buffer, but eof was already
@@ -746,8 +830,8 @@ Medium::ReadStatus Medium::Read(ref_t<bytevector> r_output)
 
     size_t pred_size = shift + m_chunk;
 
-    output.resize(pred_size);
-    int st = ReadInternal(output.data() + shift, m_chunk);
+    w_output.resize(pred_size);
+    int st = ReadInternal((w_output.data() + shift), m_chunk);
     if (st == -1)
     {
         if (IsErrorAgain())
@@ -768,22 +852,20 @@ Medium::ReadStatus Medium::Read(ref_t<bytevector> r_output)
             //
             // Set back the size this buffer had before we attempted
             // to read into it.
-            output.resize(shift);
+            w_output.resize(shift);
             return RD_DATA;
         }
-        output.clear();
+        w_output.clear();
         return RD_EOF;
     }
 
-    output.resize(shift+st);
+    w_output.resize(shift+st);
     return RD_DATA;
 }
 
-void SrtMedium::Write(ref_t<bytevector> r_buffer)
+void SrtMedium::Write(bytevector& w_buffer)
 {
-    bytevector& buffer = *r_buffer;
-
-    int st = srt_send(m_socket, buffer.data(), buffer.size());
+    int st = srt_send(m_socket, w_buffer.data(), w_buffer.size());
     if (st == SRT_ERROR)
     {
         Error(UDT::getlasterror(), "srt_send");
@@ -791,8 +873,8 @@ void SrtMedium::Write(ref_t<bytevector> r_buffer)
 
     // This should be ==, whereas > is not possible, but
     // this should simply embrace this case as a sanity check.
-    if (st >= int(buffer.size()))
-        buffer.clear();
+    if (st >= int(w_buffer.size()))
+        w_buffer.clear();
     else if (st == 0)
     {
         Error("Unexpected EOF on Write");
@@ -800,15 +882,13 @@ void SrtMedium::Write(ref_t<bytevector> r_buffer)
     else
     {
         // Remove only those bytes that were sent
-        buffer.erase(buffer.begin(), buffer.begin()+st);
+        w_buffer.erase(w_buffer.begin(), w_buffer.begin()+st);
     }
 }
 
-void TcpMedium::Write(ref_t<bytevector> r_buffer)
+void TcpMedium::Write(bytevector& w_buffer)
 {
-    bytevector& buffer = *r_buffer;
-
-    int st = ::send(m_socket, buffer.data(), buffer.size(), DEF_SEND_FLAG);
+    int st = ::send(m_socket, w_buffer.data(), w_buffer.size(), DEF_SEND_FLAG);
     if (st == -1)
     {
         Error(errno, "send");
@@ -816,8 +896,8 @@ void TcpMedium::Write(ref_t<bytevector> r_buffer)
 
     // This should be ==, whereas > is not possible, but
     // this should simply embrace this case as a sanity check.
-    if (st >= int(buffer.size()))
-        buffer.clear();
+    if (st >= int(w_buffer.size()))
+        w_buffer.clear();
     else if (st == 0)
     {
         Error("Unexpected EOF on Write");
@@ -825,7 +905,7 @@ void TcpMedium::Write(ref_t<bytevector> r_buffer)
     else
     {
         // Remove only those bytes that were sent
-        buffer.erase(buffer.begin(), buffer.begin()+st);
+        w_buffer.erase(w_buffer.begin(), w_buffer.begin()+st);
     }
 }
 
@@ -911,7 +991,7 @@ private:
                 // the one critical section - make sure no other thread
                 // is accessing it at the same time and also make join all
                 // threads that might have been accessing it. After
-                // exitting as true (meaning that it was decommissioned
+                // exiting as true (meaning that it was decommissioned
                 // as expected) it can be safely deleted.
                 if ((*i)->decommission_if_dead(main_running))
                 {
@@ -920,7 +1000,6 @@ private:
             }
         }
     }
-
 };
 
 void Tunnel::Stop()
@@ -962,8 +1041,6 @@ std::unique_ptr<Medium> main_listener;
 
 size_t default_chunk = 4096;
 
-const srt_logging::LogFA SRT_LOGFA_APP = 10;
-
 int OnINT_StopService(int)
 {
     g_tunnels.main_running = false;
@@ -985,7 +1062,7 @@ int main( int argc, char** argv )
 
     size_t chunk = default_chunk;
 
-    set<string>
+    OptionName
         o_loglevel = { "ll", "loglevel" },
         o_logfa = { "lf", "logfa" },
         o_chunk = {"c", "chunk" },
@@ -1083,7 +1160,7 @@ int main( int argc, char** argv )
             std::unique_ptr<Medium> accepted = main_listener->Accept();
             if (!g_tunnels.main_running)
             {
-                Verb() << "Service stopped. Exitting.";
+                Verb() << "Service stopped. Exiting.";
                 break;
             }
             Verb() << "Connection accepted. Connecting to the relay...";
