@@ -8,8 +8,8 @@
  * 
  */
 
-#ifndef INC__COMMON_TRANSMITMEDIA_HPP
-#define INC__COMMON_TRANSMITMEDIA_HPP
+#ifndef INC_SRT_COMMON_TRANSMITMEDIA_HPP
+#define INC_SRT_COMMON_TRANSMITMEDIA_HPP
 
 #include <string>
 #include <map>
@@ -60,7 +60,7 @@ public:
 
 protected:
 
-    void Error(UDT::ERRORINFO& udtError, string src);
+    void Error(string src);
     void Init(string host, int port, map<string,string> par, bool dir_output);
 
     virtual int ConfigurePost(SRTSOCKET sock);
@@ -93,7 +93,7 @@ public:
         // Do nothing - create just to prepare for use
     }
 
-    int Read(size_t chunk, bytevector& data, ostream& out_stats = cout) override;
+    int Read(size_t chunk, MediaPacket& pkt, ostream& out_stats = cout) override;
 
     /*
        In this form this isn't needed.
@@ -134,7 +134,7 @@ public:
     SrtTarget() {}
 
     int ConfigurePre(SRTSOCKET sock) override;
-    int Write(const char* data, size_t size, ostream &out_stats = cout) override;
+    int Write(const char* data, size_t size, int64_t src_time, ostream &out_stats = cout) override;
     bool IsOpen() override { return IsUsable(); }
     bool Broken() override { return IsBroken(); }
     void Close() override { return SrtCommon::Close(); }
@@ -183,7 +183,7 @@ public:
 
 
     SrtModel(string host, int port, map<string,string> par);
-    void Establish(ref_t<std::string> name);
+    void Establish(std::string& name);
 
     void Close()
     {
