@@ -38,13 +38,14 @@ written by
    Yunhong Gu, last updated 01/27/2011
 *****************************************************************************/
 
-#ifndef __UDT_CACHE_H__
-#define __UDT_CACHE_H__
+#ifndef INC_SRT_CACHE_H
+#define INC_SRT_CACHE_H
 
 #include <list>
 #include <vector>
 
-#include "common.h"
+#include "sync.h"
+#include "netinet_any.h"
 #include "udt.h"
 
 class CCacheItem
@@ -82,13 +83,13 @@ public:
    m_iCurrSize(0)
    {
       m_vHashPtr.resize(m_iHashSize);
-      CGuard::createMutex(m_Lock);
+      // Exception: -> CUDTUnited ctor
+      srt::sync::setupMutex(m_Lock, "Cache");
    }
 
    ~CCache()
    {
       clear();
-      CGuard::releaseMutex(m_Lock);
    }
 
 public:
@@ -98,7 +99,7 @@ public:
 
    int lookup(T* data)
    {
-      CGuard cacheguard(m_Lock);
+      srt::sync::ScopedLock cacheguard(m_Lock);
 
       int key = data->getKey();
       if (key < 0)
@@ -126,7 +127,7 @@ public:
 
    int update(T* data)
    {
-      CGuard cacheguard(m_Lock);
+      srt::sync::ScopedLock cacheguard(m_Lock);
 
       int key = data->getKey();
       if (key < 0)
@@ -223,7 +224,7 @@ private:
    int m_iHashSize;
    int m_iCurrSize;
 
-   pthread_mutex_t m_Lock;
+   srt::sync::Mutex m_Lock;
 
 private:
    CCache(const CCache&);
@@ -235,7 +236,7 @@ class CInfoBlock
 {
 public:
    uint32_t m_piIP[4];		// IP address, machine read only, not human readable format
-   int m_iIPversion;		// IP version
+   int m_iIPversion;   		// Address family: AF_INET or AF_INET6
    uint64_t m_ullTimeStamp;	// last update time
    int m_iRTT;			// RTT
    int m_iBandwidth;		// estimated bandwidth
@@ -259,7 +260,7 @@ public:
       /// @param [in] ver IP version
       /// @param [out] ip the result machine readable IP address in integer array
 
-   static void convert(const sockaddr* addr, int ver, uint32_t ip[]);
+   static void convert(const sockaddr_any& addr, uint32_t ip[4]);
 };
 
 
