@@ -50,396 +50,45 @@ modified by
    Haivision Systems Inc.
 *****************************************************************************/
 
-
-#ifndef _WIN32
-   #include <cstring>
-   #include <cerrno>
-   #include <unistd.h>
-   #if __APPLE__
-      #include "TargetConditionals.h"
-   #endif
-   #if defined(OSX) || (TARGET_OS_IOS == 1) || (TARGET_OS_TV == 1)
-      #include <mach/mach_time.h>
-   #endif
-#else
-   #include <winsock2.h>
-   #include <ws2tcpip.h>
-   #include <win/wintime.h>
-#ifndef __MINGW__
-   #include <intrin.h>
-#endif
-#endif
+#define SRT_IMPORT_TIME 1
+#include "platform_sys.h"
 
 #include <string>
 #include <sstream>
 #include <cmath>
 #include <iostream>
 #include <iomanip>
-#include "srt.h"
+#include <iterator>
+#include <vector>
+#include "udt.h"
 #include "md5.h"
 #include "common.h"
+#include "netinet_any.h"
 #include "logging.h"
+#include "packet.h"
 #include "threadname.h"
 
 #include <srt_compat.h> // SysStrError
 
-bool CTimer::m_bUseMicroSecond = false;
-uint64_t CTimer::s_ullCPUFrequency = CTimer::readCPUFrequency();
-
-pthread_mutex_t CTimer::m_EventLock = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t CTimer::m_EventCond = PTHREAD_COND_INITIALIZER;
-
-CTimer::CTimer():
-m_ullSchedTime_tk(),
-m_TickCond(),
-m_TickLock()
-{
-    pthread_mutex_init(&m_TickLock, NULL);
-
-#if ENABLE_MONOTONIC_CLOCK
-    pthread_condattr_t  CondAttribs;
-    pthread_condattr_init(&CondAttribs);
-    pthread_condattr_setclock(&CondAttribs, CLOCK_MONOTONIC);
-    pthread_cond_init(&m_TickCond, &CondAttribs);
-#else
-    pthread_cond_init(&m_TickCond, NULL);
-#endif
-}
-
-CTimer::~CTimer()
-{
-    pthread_mutex_destroy(&m_TickLock);
-    pthread_cond_destroy(&m_TickCond);
-}
-
-void CTimer::rdtsc(uint64_t &x)
-{
-   if (m_bUseMicroSecond)
-   {
-      x = getTime();
-      return;
-   }
-
-   #ifdef IA32
-      uint32_t lval, hval;
-      //asm volatile ("push %eax; push %ebx; push %ecx; push %edx");
-      //asm volatile ("xor %eax, %eax; cpuid");
-      asm volatile ("rdtsc" : "=a" (lval), "=d" (hval));
-      //asm volatile ("pop %edx; pop %ecx; pop %ebx; pop %eax");
-      x = hval;
-      x = (x << 32) | lval;
-   #elif defined(IA64)
-      asm ("mov %0=ar.itc" : "=r"(x) :: "memory");
-   #elif defined(AMD64)
-      uint32_t lval, hval;
-      asm ("rdtsc" : "=a" (lval), "=d" (hval));
-      x = hval;
-      x = (x << 32) | lval;
-   #elif defined(_WIN32)
-      // This function should not fail, because we checked the QPC
-      // when calling to QueryPerformanceFrequency. If it failed,
-      // the m_bUseMicroSecond was set to true.
-      QueryPerformanceCounter((LARGE_INTEGER *)&x);
-   #elif defined(OSX) || (TARGET_OS_IOS == 1) || (TARGET_OS_TV == 1)
-      x = mach_absolute_time();
-   #else
-      // use system call to read time clock for other archs
-      x = getTime();
-   #endif
-}
-
-uint64_t CTimer::readCPUFrequency()
-{
-   uint64_t frequency = 1;  // 1 tick per microsecond.
-
-#if defined(IA32) || defined(IA64) || defined(AMD64)
-    uint64_t t1, t2;
-
-    rdtsc(t1);
-    timespec ts;
-    ts.tv_sec = 0;
-    ts.tv_nsec = 100000000;
-    nanosleep(&ts, NULL);
-    rdtsc(t2);
-
-    // CPU clocks per microsecond
-    frequency = (t2 - t1) / 100000;
-#elif defined(_WIN32)
-    LARGE_INTEGER counts_per_sec;
-    if (QueryPerformanceFrequency(&counts_per_sec))
-        frequency = counts_per_sec.QuadPart / 1000000;
-#elif defined(OSX) || (TARGET_OS_IOS == 1) || (TARGET_OS_TV == 1)
-    mach_timebase_info_data_t info;
-    mach_timebase_info(&info);
-    frequency = info.denom * uint64_t(1000) / info.numer;
-#endif
-
-   // Fall back to microsecond if the resolution is not high enough.
-   if (frequency < 10)
-   {
-      frequency = 1;
-      m_bUseMicroSecond = true;
-   }
-   return frequency;
-}
-
-uint64_t CTimer::getCPUFrequency()
-{
-   return s_ullCPUFrequency;
-}
-
-void CTimer::sleep(uint64_t interval_tk)
-{
-   uint64_t t;
-   rdtsc(t);
-
-   // sleep next "interval" time
-   sleepto(t + interval_tk);
-}
-
-void CTimer::sleepto(uint64_t nexttime_tk)
-{
-    // Use class member such that the method can be interrupted by others
-    m_ullSchedTime_tk = nexttime_tk;
-
-    uint64_t t;
-    rdtsc(t);
-
-#if USE_BUSY_WAITING
-#if defined(_WIN32)
-    const uint64_t threshold_us = 10000;   // 10 ms on Windows: bad accuracy of timers
-#else
-    const uint64_t threshold_us = 1000;    // 1 ms on non-Windows platforms
-#endif
-#endif
-
-    while (t < m_ullSchedTime_tk)
-    {
-#if USE_BUSY_WAITING
-        uint64_t wait_us = (m_ullSchedTime_tk - t) / s_ullCPUFrequency;
-        if (wait_us <= 2 * threshold_us)
-            break;
-        wait_us -= threshold_us;
-#else
-        const uint64_t wait_us = (m_ullSchedTime_tk - t) / s_ullCPUFrequency;
-        if (wait_us == 0)
-            break;
-#endif
-
-        timespec timeout;
-#if ENABLE_MONOTONIC_CLOCK
-        clock_gettime(CLOCK_MONOTONIC, &timeout);
-        const uint64_t time_us = timeout.tv_sec * uint64_t(1000000) + (timeout.tv_nsec / 1000) + wait_us;
-        timeout.tv_sec = time_us / 1000000;
-        timeout.tv_nsec = (time_us % 1000000) * 1000;
-#else
-        timeval now;
-        gettimeofday(&now, 0);
-        const uint64_t time_us = now.tv_sec * uint64_t(1000000) + now.tv_usec + wait_us;
-        timeout.tv_sec = time_us / 1000000;
-        timeout.tv_nsec = (time_us % 1000000) * 1000;
-#endif
-
-        THREAD_PAUSED();
-        pthread_mutex_lock(&m_TickLock);
-        pthread_cond_timedwait(&m_TickCond, &m_TickLock, &timeout);
-        pthread_mutex_unlock(&m_TickLock);
-        THREAD_RESUMED();
-
-        rdtsc(t);
-    }
-
-#if USE_BUSY_WAITING
-    while (t < m_ullSchedTime_tk)
-    {
-#ifdef IA32
-        __asm__ volatile ("pause; rep; nop; nop; nop; nop; nop;");
-#elif IA64
-        __asm__ volatile ("nop 0; nop 0; nop 0; nop 0; nop 0;");
-#elif AMD64
-        __asm__ volatile ("nop; nop; nop; nop; nop;");
-#elif defined(_WIN32) && !defined(__MINGW__)
-        __nop();
-        __nop();
-        __nop();
-        __nop();
-        __nop();
-#endif
-
-        rdtsc(t);
-    }
-#endif
-}
-
-void CTimer::interrupt()
-{
-   // schedule the sleepto time to the current CCs, so that it will stop
-   rdtsc(m_ullSchedTime_tk);
-   tick();
-}
-
-void CTimer::tick()
-{
-    pthread_cond_signal(&m_TickCond);
-}
-
-uint64_t CTimer::getTime()
-{
-    // XXX Do further study on that. Currently Cygwin is also using gettimeofday,
-    // however Cygwin platform is supported only for testing purposes.
-
-    //For other systems without microsecond level resolution, add to this conditional compile
-#if defined(OSX) || (TARGET_OS_IOS == 1) || (TARGET_OS_TV == 1)
-    // Otherwise we will have an infinite recursive functions calls
-    if (m_bUseMicroSecond == false)
-    {
-        uint64_t x;
-        rdtsc(x);
-        return x / s_ullCPUFrequency;
-    }
-    // Specific fix may be necessary if rdtsc is not available either.
-    // Going further on Apple platforms might cause issue, fixed with PR #301.
-    // But it is very unlikely for the latest platforms.
-#endif
-    timeval t;
-    gettimeofday(&t, 0);
-    return t.tv_sec * uint64_t(1000000) + t.tv_usec;
-}
-
-void CTimer::triggerEvent()
-{
-    pthread_cond_signal(&m_EventCond);
-}
-
-CTimer::EWait CTimer::waitForEvent()
-{
-    timeval now;
-    timespec timeout;
-    gettimeofday(&now, 0);
-    if (now.tv_usec < 990000)
-    {
-        timeout.tv_sec = now.tv_sec;
-        timeout.tv_nsec = (now.tv_usec + 10000) * 1000;
-    }
-    else
-    {
-        timeout.tv_sec = now.tv_sec + 1;
-        timeout.tv_nsec = (now.tv_usec + 10000 - 1000000) * 1000;
-    }
-    pthread_mutex_lock(&m_EventLock);
-    int reason = pthread_cond_timedwait(&m_EventCond, &m_EventLock, &timeout);
-    pthread_mutex_unlock(&m_EventLock);
-
-    return reason == ETIMEDOUT ? WT_TIMEOUT : reason == 0 ? WT_EVENT : WT_ERROR;
-}
-
-void CTimer::sleep()
-{
-   #ifndef _WIN32
-      usleep(10);
-   #else
-      Sleep(1);
-   #endif
-}
-
-int CTimer::condTimedWaitUS(pthread_cond_t* cond, pthread_mutex_t* mutex, uint64_t delay) {
-    timeval now;
-    gettimeofday(&now, 0);
-    const uint64_t time_us = now.tv_sec * uint64_t(1000000) + now.tv_usec + delay;
-    timespec timeout;
-    timeout.tv_sec = time_us / 1000000;
-    timeout.tv_nsec = (time_us % 1000000) * 1000;
-
-    return pthread_cond_timedwait(cond, mutex, &timeout);
-}
+using namespace srt::sync;
 
 
-// Automatically lock in constructor
-CGuard::CGuard(pthread_mutex_t& lock, bool shouldwork):
-    m_Mutex(lock),
-    m_iLocked(-1)
-{
-    if (shouldwork)
-        m_iLocked = pthread_mutex_lock(&m_Mutex);
-}
-
-// Automatically unlock in destructor
-CGuard::~CGuard()
-{
-    if (m_iLocked == 0)
-        pthread_mutex_unlock(&m_Mutex);
-}
-
-// After calling this on a scoped lock wrapper (CGuard),
-// the mutex will be unlocked right now, and no longer
-// in destructor
-void CGuard::forceUnlock()
-{
-    if (m_iLocked == 0)
-    {
-        pthread_mutex_unlock(&m_Mutex);
-        m_iLocked = -1;
-    }
-}
-
-int CGuard::enterCS(pthread_mutex_t& lock)
-{
-    return pthread_mutex_lock(&lock);
-}
-
-int CGuard::leaveCS(pthread_mutex_t& lock)
-{
-    return pthread_mutex_unlock(&lock);
-}
-
-void CGuard::createMutex(pthread_mutex_t& lock)
-{
-    pthread_mutex_init(&lock, NULL);
-}
-
-void CGuard::releaseMutex(pthread_mutex_t& lock)
-{
-    pthread_mutex_destroy(&lock);
-}
-
-void CGuard::createCond(pthread_cond_t& cond)
-{
-    pthread_cond_init(&cond, NULL);
-}
-
-void CGuard::releaseCond(pthread_cond_t& cond)
-{
-    pthread_cond_destroy(&cond);
-}
-
-//
 CUDTException::CUDTException(CodeMajor major, CodeMinor minor, int err):
 m_iMajor(major),
 m_iMinor(minor)
 {
    if (err == -1)
-      #ifndef _WIN32
-         m_iErrno = errno;
-      #else
-         m_iErrno = GetLastError();
-      #endif
+       m_iErrno = NET_ERROR;
    else
       m_iErrno = err;
 }
 
-CUDTException::CUDTException(const CUDTException& e):
-m_iMajor(e.m_iMajor),
-m_iMinor(e.m_iMinor),
-m_iErrno(e.m_iErrno),
-m_strMsg()
+const char* CUDTException::getErrorMessage() const ATR_NOTHROW
 {
+    return getErrorString().c_str();
 }
 
-CUDTException::~CUDTException()
-{
-}
-
-const char* CUDTException::getErrorMessage()
+const string& CUDTException::getErrorString() const
 {
    // translate "Major:Minor" code into text message.
 
@@ -470,6 +119,10 @@ const char* CUDTException::getErrorMessage()
            m_strMsg += ": abort for security reasons";
            break;
 
+        case MN_CLOSED:
+           m_strMsg += ": socket closed during operation";
+           break;
+
         default:
            break;
         }
@@ -506,6 +159,11 @@ const char* CUDTException::getErrorMessage()
            m_strMsg += ": unable to allocate buffers";
            break;
 
+
+        case MN_OBJECT:
+           m_strMsg += ": unable to allocate system object";
+           break;
+
         default:
            break;
         }
@@ -596,6 +254,9 @@ const char* CUDTException::getErrorMessage()
            m_strMsg += ": Invalid epoll ID";
            break;
 
+        case MN_EEMPTY:
+           m_strMsg += ": All sockets removed from epoll, waiting would deadlock";
+
         default:
            break;
         }
@@ -645,7 +306,7 @@ const char* CUDTException::getErrorMessage()
       m_strMsg += ": " + SysStrError(m_iErrno);
    }
 
-   return m_strMsg.c_str();
+   return m_strMsg;
 }
 
 #define UDT_XCODE(mj, mn) (int(mj)*1000)+int(mn)
@@ -699,39 +360,101 @@ bool CIPAddress::ipcmp(const sockaddr* addr1, const sockaddr* addr2, int ver)
    return false;
 }
 
-void CIPAddress::ntop(const sockaddr* addr, uint32_t ip[4], int ver)
+void CIPAddress::ntop(const sockaddr_any& addr, uint32_t ip[4])
 {
-   if (AF_INET == ver)
-   {
-      sockaddr_in* a = (sockaddr_in*)addr;
-      ip[0] = a->sin_addr.s_addr;
-   }
-   else
-   {
-      sockaddr_in6* a = (sockaddr_in6*)addr;
+    if (addr.family() == AF_INET)
+    {
+        ip[0] = addr.sin.sin_addr.s_addr;
+    }
+    else
+    {
+      const sockaddr_in6* a = &addr.sin6;
       ip[3] = (a->sin6_addr.s6_addr[15] << 24) + (a->sin6_addr.s6_addr[14] << 16) + (a->sin6_addr.s6_addr[13] << 8) + a->sin6_addr.s6_addr[12];
       ip[2] = (a->sin6_addr.s6_addr[11] << 24) + (a->sin6_addr.s6_addr[10] << 16) + (a->sin6_addr.s6_addr[9] << 8) + a->sin6_addr.s6_addr[8];
       ip[1] = (a->sin6_addr.s6_addr[7] << 24) + (a->sin6_addr.s6_addr[6] << 16) + (a->sin6_addr.s6_addr[5] << 8) + a->sin6_addr.s6_addr[4];
       ip[0] = (a->sin6_addr.s6_addr[3] << 24) + (a->sin6_addr.s6_addr[2] << 16) + (a->sin6_addr.s6_addr[1] << 8) + a->sin6_addr.s6_addr[0];
-   }
+    }
 }
 
-void CIPAddress::pton(sockaddr* addr, const uint32_t ip[4], int ver)
+// XXX This has void return and the first argument is passed by reference.
+// Consider simply returning sockaddr_any by value.
+void CIPAddress::pton(sockaddr_any& w_addr, const uint32_t ip[4], int agent_family, const sockaddr_any& peer)
 {
-   if (AF_INET == ver)
+   if (agent_family == AF_INET)
    {
-      sockaddr_in* a = (sockaddr_in*)addr;
+      sockaddr_in* a = (&w_addr.sin);
       a->sin_addr.s_addr = ip[0];
    }
-   else
+   else // AF_INET6
    {
-      sockaddr_in6* a = (sockaddr_in6*)addr;
-      for (int i = 0; i < 4; ++ i)
+      // Check if the peer address is a model of IPv4-mapped-on-IPv6.
+      // If so, it means that the `ip` array should be interpreted as IPv4.
+
+      uint16_t* peeraddr16 = (uint16_t*)peer.sin6.sin6_addr.s6_addr;
+      static const uint16_t ipv4on6_model [8] =
       {
-         a->sin6_addr.s6_addr[i * 4] = ip[i] & 0xFF;
-         a->sin6_addr.s6_addr[i * 4 + 1] = (unsigned char)((ip[i] & 0xFF00) >> 8);
-         a->sin6_addr.s6_addr[i * 4 + 2] = (unsigned char)((ip[i] & 0xFF0000) >> 16);
-         a->sin6_addr.s6_addr[i * 4 + 3] = (unsigned char)((ip[i] & 0xFF000000) >> 24);
+          0, 0, 0, 0, 0, 0xFFFF, 0, 0
+      };
+
+      // Compare only first 6 words. Remaining 2 words
+      // comprise the IPv4 address, if these first 6 match.
+      const uint16_t* mbegin = ipv4on6_model;
+      const uint16_t* mend = ipv4on6_model + 6;
+
+      bool is_mapped_ipv4 = (std::mismatch(mbegin, mend, peeraddr16).first == mend);
+
+      sockaddr_in6* a = (&w_addr.sin6);
+
+      if (!is_mapped_ipv4)
+      {
+          // Here both agent and peer use IPv6, in which case
+          // `ip` contains the full IPv6 address, so just copy
+          // it as is.
+
+          // XXX Possibly, a simple
+          // memcpy( (a->sin6_addr.s6_addr), ip, 16);
+          // would do the same thing, and faster. The address in `ip`,
+          // even though coded here as uint32_t, is still big endian.
+          for (int i = 0; i < 4; ++ i)
+          {
+              a->sin6_addr.s6_addr[i * 4 + 0] = ip[i] & 0xFF;
+              a->sin6_addr.s6_addr[i * 4 + 1] = (unsigned char)((ip[i] & 0xFF00) >> 8);
+              a->sin6_addr.s6_addr[i * 4 + 2] = (unsigned char)((ip[i] & 0xFF0000) >> 16);
+              a->sin6_addr.s6_addr[i * 4 + 3] = (unsigned char)((ip[i] & 0xFF000000) >> 24);
+          }
+      }
+      else // IPv4 mapped on IPv6
+      {
+          // Here agent uses IPv6 with IPPROTO_IPV6/IPV6_V6ONLY == 0
+          // In this case, the address in `ip` uses only the `ip[0]`
+          // element carrying the IPv4 address, rest of the elements
+          // is 0. This address must be interpreted as IPv4, but set
+          // as the IPv4-on-IPv6 address.
+          //
+          // Unfortunately, sockaddr_in6 doesn't give any straightforward
+          // method for it, although the official size of a single element
+          // of the IPv6 address is 16-bit.
+
+          memset((a->sin6_addr.s6_addr), 0, sizeof a->sin6_addr.s6_addr);
+
+          // There are also available sin6_addr.s6_addr32, but
+          // this is kinda nonportable.
+          uint32_t* paddr32 = (uint32_t*)a->sin6_addr.s6_addr;
+          uint16_t* paddr16 = (uint16_t*)a->sin6_addr.s6_addr;
+
+          // layout: of IPv4 address 192.168.128.2
+          // 16-bit:
+          // [0000: 0000: 0000: 0000: 0000: FFFF: 192.168:128.2]
+          // 8-bit
+          // [00/00/00/00/00/00/00/00/00/00/FF/FF/192/168/128/2]
+          // 32-bit
+          // [00000000 && 00000000 && 0000FFFF && 192.168.128.2]
+
+          // Spreading every 16-bit word separately to avoid endian dilemmas
+          paddr16[2 * 2 + 0] = 0;
+          paddr16[2 * 2 + 1] = 0xFFFF;
+
+          paddr32[3] = ip[0]; // IPv4 address encoded here
       }
    }
 }
@@ -824,7 +547,9 @@ std::string MessageTypeStr(UDTMessageType mt, uint32_t extt)
         "EXT:kmreq",
         "EXT:kmrsp",
         "EXT:sid",
-        "EXT:congctl"
+        "EXT:congctl",
+        "EXT:filter",
+        "EXT:group"
     };
 
 
@@ -890,42 +615,113 @@ extern const char* const srt_rejectreason_msg [] = {
     "Password required or unexpected",
     "MessageAPI/StreamAPI collision",
     "Congestion controller type collision",
-    "Packet Filter type collision"
+    "Packet Filter type collision",
+    "Group settings collision",
+    "Connection timeout"
 };
 
-const char* srt_rejectreason_str(SRT_REJECT_REASON rid)
+const char* srt_rejectreason_str(int id)
 {
-    int id = rid;
+    if (id >= SRT_REJC_PREDEFINED)
+    {
+        return "Application-defined rejection reason";
+    }
+
     static const size_t ra_size = Size(srt_rejectreason_msg);
     if (size_t(id) >= ra_size)
         return srt_rejectreason_msg[0];
     return srt_rejectreason_msg[id];
 }
 
+bool SrtParseConfig(string s, SrtConfig& w_config)
+{
+    using namespace std;
+
+    vector<string> parts;
+    Split(s, ',', back_inserter(parts));
+
+    w_config.type = parts[0];
+
+    for (vector<string>::iterator i = parts.begin()+1; i != parts.end(); ++i)
+    {
+        vector<string> keyval;
+        Split(*i, ':', back_inserter(keyval));
+        if (keyval.size() != 2)
+            return false;
+        w_config.parameters[keyval[0]] = keyval[1];
+    }
+
+    return true;
+}
+
+uint64_t PacketMetric::fullBytes()
+{
+    static const int PKT_HDR_SIZE = CPacket::HDR_SIZE + CPacket::UDP_HDR_SIZE;
+    return bytes + pkts * PKT_HDR_SIZE;
+}
+
+
 // Some logging imps
 #if ENABLE_LOGGING
 
 namespace srt_logging
 {
 
-std::string FormatTime(uint64_t time)
+
+std::string SockStatusStr(SRT_SOCKSTATUS s)
 {
-    using namespace std;
+    if (int(s) < int(SRTS_INIT) || int(s) > int(SRTS_NONEXIST))
+        return "???";
 
-    time_t sec = time/1000000;
-    time_t usec = time%1000000;
+    static struct AutoMap
+    {
+        // Values start from 1, so do -1 to avoid empty cell
+        std::string names[int(SRTS_NONEXIST)-1+1];
 
-    time_t tt = sec;
-    struct tm tm = SysLocalTime(tt);
+        AutoMap()
+        {
+#define SINI(statename) names[SRTS_##statename-1] = #statename
+            SINI(INIT);
+            SINI(OPENED);
+            SINI(LISTENING);
+            SINI(CONNECTING);
+            SINI(CONNECTED);
+            SINI(BROKEN);
+            SINI(CLOSING);
+            SINI(CLOSED);
+            SINI(NONEXIST);
+#undef SINI
+        }
+    } names;
 
-    char tmp_buf[512];
-    strftime(tmp_buf, 512, "%X.", &tm);
-
-    ostringstream out;
-    out << tmp_buf << setfill('0') << setw(6) << usec;
-    return out.str();
+    return names.names[int(s)-1];
 }
 
+#if ENABLE_EXPERIMENTAL_BONDING
+std::string MemberStatusStr(SRT_MEMBERSTATUS s)
+{
+    if (int(s) < int(SRT_GST_PENDING) || int(s) > int(SRT_GST_BROKEN))
+        return "???";
+
+    static struct AutoMap
+    {
+        std::string names[int(SRT_GST_BROKEN)+1];
+
+        AutoMap()
+        {
+#define SINI(statename) names[SRT_GST_##statename] = #statename
+            SINI(PENDING);
+            SINI(IDLE);
+            SINI(RUNNING);
+            SINI(BROKEN);
+#undef SINI
+        }
+    } names;
+
+    return names.names[int(s)];
+}
+#endif
+
 LogDispatcher::Proxy::Proxy(LogDispatcher& guy) : that(guy), that_enabled(that.CheckEnabled())
 {
     if (that_enabled)
